@@ -1,5 +1,5 @@
 package Date::Manip::Delta;
-# Copyright (c) 1995-2013 Sullivan Beck. All rights reserved.
+# Copyright (c) 1995-2019 Sullivan Beck. All rights reserved.
 # This program is free software; you can redistribute it and/or modify it
 # under the same terms as Perl itself.
 
@@ -25,7 +25,7 @@ use Date::Manip::Base;
 use Date::Manip::TZ;
 
 our $VERSION;
-$VERSION='6.39';
+$VERSION='6.76';
 END { undef $VERSION; }
 
 ########################################################################
@@ -56,50 +56,59 @@ sub _init {
    my $dmb = $$dmt{'base'};
 
    $$self{'err'}  = '';
-   $$self{'data'} = {
-                     'delta'      => $def,  # the delta (all negative fields signed)
-                     'in'         => '',    # the string that was parsed (if any)
-                     'length'     => 0,     # length of delta (in seconds)
+   $$self{'data'} =
+     {
+      'delta'      => $def,        # the delta put in (all negative fields signed)
 
-                     'gotmode'    => 0,     # 1 if mode set explicitly
-                     'business'   => 0,     # 1 for a business delta
+      'in'         => '',          # the string that was parsed (if any)
+      'length'     => 0,           # length of delta (in seconds)
 
-                     'f'          => {},    # format fields
-                     'flen'       => {},    # field lengths
-                    }
+      'gotmode'    => 0,           # 1 if mode set explicitly
+      'mode'       => 'standard',  # standard/business
+      'type'       => 'exact',     # exact, semi, estimated, approx
+      'type_from'  => 'init',      # where did the type come from
+                                   #    init - from here
+                                   #    opt  - specified in an option/string
+                                   #    det  - determined automatically
+      'normalized' => 1,           # 1 if normalized
+
+      'f'          => {},          # format fields
+      'flen'       => {},          # field lengths
+     }
 }
 
 sub _init_args {
    my($self) = @_;
 
    my @args = @{ $$self{'args'} };
-   if (@args) {
-      if ($#args == 0) {
-         $self->parse($args[0]);
-      } else {
-         warn "WARNING: [new] invalid arguments: @args\n";
-      }
-   }
+   $self->parse(@args);
 }
 
 sub value {
-   my($self) = @_;
+   my($self,$as_input) = @_;
+
+   if ($$self{'err'}) {
+      return ()  if (wantarray);
+      return '';
+   }
+
    my $dmt = $$self{'tz'};
    my $dmb = $$dmt{'base'};
 
-   return undef  if ($$self{'err'});
-   if (wantarray) {
-      return @{ $$self{'data'}{'delta'} };
-   } else {
-      my @delta = @{ $$self{'data'}{'delta'} };
-      my $err;
-      ($err,@delta) = $dmb->_delta_fields( { 'nonorm'  => 1,
-                                             'source'  => 'delta',
-                                             'sign'    => 0 },
-                                           [@delta]);
-      return undef  if ($err);
-      return join(':',@delta);
-   }
+   my @delta   = @{ $$self{'data'}{'delta'} };
+
+   return @delta  if (wantarray);
+   my $err;
+
+   my %o = ( 'source'  => 'delta',
+             'nonorm'  => 1,
+             'type'    => $$self{'data'}{'type'},
+             'sign'    => 0,
+             'mode'    => $$self{'data'}{'mode'},
+           );
+
+   ($err,@delta) = $dmb->_delta_fields( \%o, [@delta]);
+   return join(':',@delta);
 }
 
 sub input {
@@ -112,87 +121,192 @@ sub input {
 ########################################################################
 
 BEGIN {
-   my %ops = map { $_,1 } qw( delta business normal standard );
    my %f   = qw( y 0  M 1  w 2  d 3  h 4  m 5  s 6 );
 
    sub set {
-      my($self,$field,$val,$no_normalize) = @_;
-
-      my $dmt        = $$self{'tz'};
-      my $dmb        = $$dmt{'base'};
-      my $zone       = $$self{'data'}{'tz'};
-      my $gotmode    = $$self{'data'}{'gotmode'};
-      my $business   = 0;
-
-      my (@delta,$err);
-
-      if (exists $ops{lc($field)}) {
-         $field       = lc($field);
-
-         if ($field eq 'business') {
-            $business = 1;
-            $gotmode  = 1;
-         } elsif ($field eq 'normal'  ||  $field eq 'standard') {
-            $business = 0;
-            $gotmode  = 1;
-         } elsif ($field eq 'delta') {
-            $business = $$self{'data'}{'business'};
-            $gotmode  = $$self{'data'}{'gotmode'};
+      my($self,@args) = @_;
+      my %opts;
+      if      (ref($args[0]) eq 'HASH') {
+         %opts = %{ $args[0] };
+      } else {
+         # *** DEPRECATED 7.0 ***
+         if (@args == 3) {
+            %opts = ( $args[0] => $args[1],
+                      'nonorm' => ($args[2] ? 1 : 0) );
+         } else {
+            %opts = ( $args[0] => $args[1] );
          }
+      }
 
-         ($err,@delta) = $dmb->_delta_fields( { 'nonorm'   => $no_normalize,
-                                                'business' => $business,
-                                                'source'   => 'delta',
-                                                'sign'     => -1 },
-                                              $val);
+      # Check for some invalid opts
 
-      } elsif (exists $f{$field}) {
+      foreach my $key (keys %opts) {
+         my $val = $opts{$key};
+         delete $opts{$key};
 
-         if ($$self{'err'}) {
-            $$self{'err'} = "[set] Invalid delta";
-            return 1;
-         }
+         # *** DEPRECATED 7.0 ***
+         $key    = 'standard'  if (lc($key) eq 'normal');
 
-         @delta             = @{ $$self{'data'}{'delta'} };
-         $business          = $$self{'data'}{'business'};
-         $delta[$f{$field}] = $val;
+         if (lc($key) eq 'delta'     ||
+             lc($key) eq 'business'  ||
+             lc($key) eq 'standard'  ||
+             lc($key) eq 'nonorm'    ||
+             lc($key) eq 'mode'      ||
+             lc($key) eq 'type') {
 
-         ($err,@delta) = $dmb->_delta_fields( { 'nonorm'   => $no_normalize,
-                                                'business' => $business,
-                                                'source'   => 'delta',
-                                                'sign'     => -1 },
-                                              [@delta]);
+            if (exists $opts{lc($key)}) {
+               $key = lc($key);
+               $$self{'err'} = "[set] Invalid option: $key entered twice";
+               return 1;
+            }
 
-      } elsif (lc($field) eq 'mode') {
+            $opts{lc($key)} = $val;
 
-         @delta             = @{ $$self{'data'}{'delta'} };
-         $val               = lc($val);
-         if ($val eq 'business'  ||  $val eq 'normal'  ||  $val eq 'standard') {
-            $gotmode        = 1;
-            $business       = ($val eq 'business' ? 1 : 0);
+         } elsif ($key =~ /^[yMwdhms]$/) {
+
+            $opts{$key} = $val;
 
          } else {
-            $$self{'err'} = "[set] Invalid mode: $val";
+            $$self{'err'} = "[set] Unknown option: $key";
+            return 1;
+         }
+      }
+
+      if ( (exists $opts{'delta'}) +
+           (exists $opts{'business'}) +
+           (exists $opts{'standard'}) +
+           (exists $opts{'y'} || exists $opts{'M'} || exists $opts{'w'} ||
+            exists $opts{'d'} || exists $opts{'h'} || exists $opts{'m'} ||
+            exists $opts{'s'})
+           > 1 ) {
+         $$self{'err'} = "[set] Fields set multiple times";
+         return 1;
+      }
+
+      if (exists $opts{'mode'}  &&  $opts{'mode'} !~ /^(business|standard)$/) {
+         $$self{'err'} = "[set] Unknown value for mode: $opts{mode}";
+         return 1;
+      }
+      if (exists $opts{'type'}  &&
+          $opts{'type'} !~ /^(exact|semi|estimated|approx)$/) {
+         $$self{'err'} = "[set] Unknown value for type: $opts{type}";
+         return 1;
+      }
+
+      if ( (exists $opts{'business'}) +
+           (exists $opts{'standard'}) +
+           (exists $opts{'mode'})
+           > 1 ) {
+         $$self{'err'} = "[set] Mode set multiple times";
+         return 1;
+      } elsif (exists $opts{'business'}) {
+         $opts{'delta'} = $opts{'business'};
+         $opts{'mode'}  = 'business';
+      } elsif (exists $opts{'standard'}) {
+         $opts{'delta'} = $opts{'standard'};
+         $opts{'mode'}  = 'standard';
+      }
+
+      # If we are setting delta/business/standard, we need to initialize
+      # all the parameters.
+
+      my @delta;
+      if (exists $opts{'delta'}) {
+         if (ref($opts{'delta'}) ne 'ARRAY') {
+            $$self{'err'} = "[set] Option delta requires an array value";
             return 1;
          }
 
+         # Init everything because we're setting an entire new delta
+         $self->_init();
+         @delta = @{ $opts{'delta'} };
+
       } else {
-
-         $$self{'err'} = "[set] Invalid field: $field";
-         return 1;
-
+         @delta      = @{ $$self{'data'}{'delta'} };
       }
 
+      # Figure out the parameters.  Include the nonorm/mode/type
+      # options.
+
+      my $err;
+      my $dmt        = $$self{'tz'};
+      my $dmb        = $$dmt{'base'};
+      my $gotmode    = (exists $opts{'mode'} ? 1 : $$self{'data'}{'gotmode'});
+      my $mode       = (exists $opts{'mode'} ? $opts{'mode'} :
+                        $$self{'data'}{'mode'});
+      my $nonorm     = (exists $opts{'nonorm'} ? $opts{'nonorm'} : 0);
+
+      my ($type,$type_from);
+      if (exists $opts{'type'}) {
+         $type       = $opts{'type'};
+         $type_from  = 'opt';
+      } else {
+         $type       = $$self{'data'}{'type'};
+         $type_from  = $$self{'data'}{'type_from'};
+      }
+
+      # If we're setting individual fields, do that now
+
+      {
+         my $field_set = 0;
+
+         # Check all individual fields
+         foreach my $opt (qw(y M w d h m s)) {
+            if (exists $opts{$opt}) {
+               if (ref($opts{$opt})) {
+                  $$self{'err'} = "[set] Option $opt requires a scalar value";
+                  return 1;
+               }
+               my $val = $opts{$opt};
+               if (! $dmb->_is_num($val)) {
+                  $$self{'err'} = "[set] Option $opt requires a numerical value";
+                  return 1;
+               }
+               $delta[ $f{$opt} ] = $val;
+               $field_set         = 1;
+            }
+         }
+
+         # If none were set, than we're done with setting.
+         last  if (! $field_set);
+
+         if ($$self{'err'}) {
+            return 1;
+         }
+      }
+
+      # Check that the type is consistent with @delta.
+
+      ($err,$type,$type_from) =
+        $dmb->_check_delta_type($mode,$type,$type_from,@delta);
+
       if ($err) {
-         $$self{'err'} = "[set] Invalid field value: $field";
+         $$self{'err'} = "[set] $err";
+         return 1;
+      }
+
+      my %o = ( 'source'  => 'delta',
+                'nonorm'  => $nonorm,
+                'type'    => $type,
+                'sign'    => -1,
+                'mode'    => $mode,
+              );
+
+      ($err,@delta) = $dmb->_delta_fields( \%o, [@delta]);
+
+      if ($err) {
+         $$self{'err'} = "[set] $err";
          return 1;
       }
 
-      $self->_init();
       $$self{'data'}{'delta'}      = [ @delta ];
-      $$self{'data'}{'business'}   = $business;
+      $$self{'data'}{'mode'}       = $mode;
       $$self{'data'}{'gotmode'}    = $gotmode;
+      $$self{'data'}{'type'}       = $type;
+      $$self{'data'}{'type_from'}  = $type_from;
+      $$self{'data'}{'normalized'} = 1-$nonorm;
       $$self{'data'}{'length'}     = 'unknown';
+      $$self{'data'}{'in'}         = '';
 
       return 0;
    }
@@ -248,56 +362,45 @@ sub _rx {
 
 sub parse {
    my($self,$instring,@args) = @_;
-   my($business,$no_normalize,$gotmode,$err,@delta);
+   $self->_init();
 
-   if (@args == 2) {
-      ($business,$no_normalize) = (lc($args[0]),lc($args[1]));
-      if      ($business eq 'standard') {
-         $business = 0;
-      } elsif ($business eq 'business') {
-         $business = 1;
-      } elsif ($business) {
-         $business = 1;
-      } else {
-         $business = 0;
-      }
-      if ($no_normalize) {
-         $no_normalize = 1;
-      } else {
-         $no_normalize = 0;
-      }
-      $gotmode = 1;
+   my %opts;
+   if (ref($args[0]) eq 'HASH') {
+      %opts = %{ $args[0] };
 
-   } elsif (@args == 1) {
-      my $arg = lc($args[0]);
-      if      ($arg eq 'standard') {
-         $business     = 0;
-         $no_normalize = 0;
-         $gotmode      = 1;
-      } elsif ($arg eq 'business') {
-         $business     = 1;
-         $no_normalize = 0;
-         $gotmode      = 1;
-      } elsif ($arg eq 'nonormalize') {
-         $business     = 0;
-         $no_normalize = 1;
-         $gotmode      = 0;
-      } elsif ($arg) {
-         $business     = 1;
-         $no_normalize = 0;
-         $gotmode      = 1;
-      } else {
-         $business     = 0;
-         $no_normalize = 0;
-         $gotmode      = 0;
-      }
-   } elsif (@args == 0) {
-      $business     = 0;
-      $no_normalize = 0;
-      $gotmode      = 0;
    } else {
-      $$self{'err'} = "[parse] Unknown arguments";
-      return 1;
+      # *** DEPRECATED 7.0 ***
+
+      my($business,$no_normalize);
+
+      if (@args == 2) {
+         ($business,$no_normalize) = (lc($args[0]),lc($args[1]));
+         if      ($business eq 'standard'  ||  ! $business) {
+            $opts{'mode'} = 'standard';
+         } else {
+            $opts{'mode'} = 'business';
+         }
+
+         $opts{'nonorm'} = ($no_normalize ? 1 : 0);
+
+      } elsif (@args == 1) {
+         my $arg = lc($args[0]);
+         if      ($arg eq 'standard') {
+            $opts{'mode'} = 'standard';
+         } elsif ($arg eq 'business') {
+            $opts{'mode'} = 'business';
+         } elsif ($arg eq 'nonormalize') {
+            $opts{'nonorm'} = 1;
+         } elsif ($arg) {
+            $opts{'mode'} = 'business';
+         } else {
+            $opts{'mode'} = 'standard';
+         }
+
+      } elsif (@args) {
+         $$self{'err'} = "[parse] Unknown arguments";
+         return 1;
+      }
    }
 
    my $dmt = $$self{'tz'};
@@ -311,13 +414,18 @@ sub parse {
 
    #
    # Parse the string
+   #    $err   : any error
+   #    @delta : the delta parsed
+   #    $mode  : the mode string (if any) in the string
    #
 
+   my ($err,@delta,$mode);
+   $mode         = '';
    $$self{'err'} = '';
    $instring     =~ s/^\s*//;
    $instring     =~ s/\s*$//;
 
- PARSE: {
+   PARSE: {
 
       # First, we'll try the standard format (without a mode string)
 
@@ -330,40 +438,25 @@ sub parse {
 
       my @strings = $dmb->_encoding($instring);
       my $moderx  = $self->_rx('mode');
-      my %mode    = ();
 
       foreach my $string (@strings) {
          if ($string =~ s/\s*$moderx\s*//i) {
-            my $b = $1;
-            if ($$dmb{'data'}{'wordmatch'}{'mode'}{lc($b)} == 1) {
-               $b = 0;
+            my $m = $1;
+            if ($$dmb{'data'}{'wordmatch'}{'mode'}{lc($m)} == 1) {
+               $m = 'standard';
             } else {
-               $b = 1;
+               $m = 'business';
             }
+            $mode = $m;
 
             ($err,@delta) = $dmb->_split_delta($string);
-            if (! $err) {
-               $business = $b;
-               $gotmode  = 1;
-               last PARSE;
-            }
-
-            $mode{$string} = $b;
+            last PARSE  if (! $err);
          }
       }
 
       # Now we'll check each string for an expanded form delta.
 
       foreach my $string (@strings) {
-         my($b,$g);
-         if (exists $mode{$string}) {
-            $b = $mode{$string};
-            $g = 1;
-         } else {
-            $b = $business;
-            $g = 0;
-         }
-
          my $past    = 0;
 
          my $whenrx  = $self->_rx('when');
@@ -378,8 +471,6 @@ sub parse {
          my $rx        = $self->_rx('expanded');
          if ($string  &&
              $string   =~ $rx) {
-            $business  = $b;
-            $gotmode   = $g;
             @delta     = @+{qw(y m w d h mn s)};
             foreach my $f (@delta) {
                if (! defined $f) {
@@ -408,22 +499,43 @@ sub parse {
       return 1;
    }
 
-   ($err,@delta) = $dmb->_delta_fields( { 'nonorm'   => $no_normalize,
-                                          'business' => $business,
-                                          'source'   => 'string',
-                                          'sign'     => -1 },
-                                        [@delta]);
+   # If the string contains a mode string and the mode was passed in
+   # as an option, they must be identical.
+
+   if ($mode  &&  exists($opts{'mode'})  &&  $mode ne $opts{'mode'}) {
+      $$self{'err'} =
+        "[parse] Mode option conflicts with mode specified in string";
+      return 1;
+   }
+   $mode = $opts{'mode'}  if (exists $opts{'mode'});
+   $mode = 'standard'     if (! $mode);
+
+   # Figure out the type.
+
+   my %o = ( 'source'  => 'string',
+             'nonorm'  => (exists $opts{'nonorm'} ? $opts{'nonorm'} : 0),
+             'sign'    => -1,
+             'mode'    => $mode,
+           );
+
+   ($err,@delta) = $dmb->_delta_fields( \%o, [@delta]);
+   my $type      = $o{'type'};
+   my $type_from = $o{'type_from'};
 
    if ($err) {
-      $$self{'err'} = "[parse] Invalid delta string";
+      $$self{'err'} = "[parse] $err";
       return 1;
    }
 
    $$self{'data'}{'in'}         = $instring;
    $$self{'data'}{'delta'}      = [@delta];
-   $$self{'data'}{'business'}   = $business;
-   $$self{'data'}{'gotmode'}    = $gotmode;
+   $$self{'data'}{'mode'}       = $mode;
+   $$self{'data'}{'gotmode'}    = ($mode  ||  exists $opts{'mode'} ? 1 : 0);
+   $$self{'data'}{'type'}       = $type;
+   $$self{'data'}{'type_from'}  = $type_from;
    $$self{'data'}{'length'}     = 'unknown';
+   $$self{'data'}{'normalized'} = ($opts{'nonorm'} ? 0 : 1);
+
    return 0;
 }
 
@@ -568,7 +680,7 @@ sub _printf_delta {
    #   where [SETx] is a listref of fields from one set of fields
 
    my @set;
-   my $business = $$self{'data'}{'business'};
+   my $mode = $$self{'data'}{'mode'};
 
    my $f0 = $tmp{$field0};
    my $f1 = $tmp{$field1};
@@ -576,20 +688,26 @@ sub _printf_delta {
    if ($field0 eq $field1) {
       @set = ( [ $delta[$f0] ] );
 
-   } elsif ($business) {
+   } elsif ($mode eq 'business') {
 
       if ($f0 <= 1) {
          # if (field0 = y or M)
          #    add [y,M]
-         #    field0 = w   OR   done if field1 = M
-         push(@set, [ @delta[0..1] ]);
+         #    if field1 = M
+         #       done
+         #    else
+         #       field0 = w
+         push(@set, [ @delta[$f0..1] ]);
          $f0 = ($f1 == 1 ? 7 : 2);
       }
 
       if ($f0 == 2) {
          # if (field0 = w)
          #    add [w]
-         #    field0 = d  OR  done if field1 = w
+         #    if field1 = w
+         #       done
+         #    else
+         #       field0 = d
          push(@set, [ $delta[2] ]);
          $f0 = ($f1 == 2 ? 7 : 3);
       }
@@ -603,11 +721,31 @@ sub _printf_delta {
       if ($f0 <= 1) {
          # if (field0 = y or M)
          #    add [y,M]
-         #    field0 = w   OR   done if field1 = M
-         push(@set, [ @delta[0..1] ]);
+         #    if field1 = M
+         #       done
+         #    else
+         #       field0 = w
+         push(@set, [ @delta[$f0..1] ]);
          $f0 = ($f1 == 1 ? 7 : 2);
       }
 
+      if ($f0 <= 3) {
+         # if (field0 = w or d)
+         #    if (field1 = w or d)
+         #       add [w ... [f1]]
+         #       done
+         #    else
+         #       add [w,d]
+         #       field0 = h
+         if ($f1 <= 3) {
+            push(@set, [ @delta[$f0..$f1] ]);
+            $f0 = 7;
+         } else {
+            push(@set, [ @delta[$f0..3] ]);
+            $f0 = 4;
+         }
+      }
+
       if ($f0 <= 6) {
          push(@set, [ @delta[$f0..$f1] ]);
       }
@@ -681,14 +819,18 @@ sub _printf_field {
    if ($width) {
       if      ($pad eq ">") {
          $val = "$s$val";
-         $val .= ' 'x($width-length($val));
+         my $pad = ($width > length($val) ? $width - length($val) : 0);
+         $val .= ' 'x$pad;
 
       } elsif ($pad eq "<") {
          $val = "$s$val";
-         $val = ' 'x($width-length($val)) . $val;
+         my $pad = ($width > length($val) ? $width - length($val) : 0);
+         $val = ' 'x$pad . $val;
 
       } else {
-         $val = $s . '0'x($width-length($val)-length($s)) . $val;
+         my $pad = ($width > length($val)-length($s) ?
+                    $width - length($val) - length($s): 0);
+         $val = $s . '0'x$pad . $val;
       }
    } else {
       $val = "$s$val";
@@ -724,15 +866,15 @@ sub _printf_field_val {
    # Find the length of 1 unit of each field in terms of seconds.
 
    if (! exists $$self{'data'}{'flen'}{'s'}) {
-      my $business = $$self{'data'}{'business'};
+      my $mode     = $$self{'data'}{'mode'};
       my $dmb      = $self->base();
       $$self{'data'}{'flen'} = { 's'  => 1,
                                  'm'  => 60,
                                  'h'  => 3600,
-                                 'd'  => $$dmb{'data'}{'len'}{$business}{'dl'},
-                                 'w'  => $$dmb{'data'}{'len'}{$business}{'wl'},
-                                 'M'  => $$dmb{'data'}{'len'}{$business}{'ml'},
-                                 'y'  => $$dmb{'data'}{'len'}{$business}{'yl'},
+                                 'd'  => $$dmb{'data'}{'len'}{$mode}{'dl'},
+                                 'w'  => $$dmb{'data'}{'len'}{$mode}{'wl'},
+                                 'M'  => $$dmb{'data'}{'len'}{$mode}{'ml'},
+                                 'y'  => $$dmb{'data'}{'len'}{$mode}{'yl'},
                                };
    }
 
@@ -768,35 +910,16 @@ sub type {
    my($self,$op) = @_;
    $op = lc($op);
 
-   if      ($op eq 'business') {
-      return $$self{'data'}{'business'};
-   } elsif ($op eq 'standard') {
-      return 1-$$self{'data'}{'business'};
+   if ($op eq 'business'  ||
+       $op eq 'standard') {
+      return ($$self{'data'}{'mode'} eq $op ? 1 : 0);
    }
 
-   my($exact,$semi,$approx) = (0,0,0);
-   my($y,$m,$w,$d,$h,$mn,$s) = @{ $$self{'data'}{'delta'} };
-   if ($y  ||  $m) {
-      $approx = 1;
-   } elsif ($w  ||  (! $$self{'data'}{'business'}  &&  $d)) {
-      $semi = 1;
-   } else {
-      $exact = 1;
-   }
-
-   if      ($op eq 'exact') {
-      return $exact;
-   } elsif ($op eq 'semi') {
-      return $semi;
-   } elsif ($op eq 'approx') {
-      return $approx;
-   }
-
-   return undef;
+   return ($$self{'data'}{'type'} eq $op ? 1 : 0);
 }
 
 sub calc {
-   my($self,$obj,$subtract,$no_normalize) = @_;
+   my($self,$obj,@args) = @_;
    if ($$self{'err'}) {
       $$self{'err'} = "[calc] First object invalid (delta)";
       return undef;
@@ -807,14 +930,14 @@ sub calc {
          $$self{'err'} = "[calc] Second object invalid (date)";
          return undef;
       }
-      return $obj->calc($self,$subtract);
+      return $obj->calc($self,@args);
 
    } elsif (ref($obj) eq 'Date::Manip::Delta') {
       if ($$obj{'err'}) {
          $$self{'err'} = "[calc] Second object invalid (delta)";
          return undef;
       }
-      return $self->_calc_delta_delta($obj,$subtract,$no_normalize);
+      return $self->_calc_delta_delta($obj,@args);
 
    } else {
       $$self{'err'} = "[calc] Second object must be a Date/Delta object";
@@ -822,38 +945,46 @@ sub calc {
    }
 }
 
+sub __type_max {
+   my($type1,$type2) = @_;
+   return $type1  if ($type1 eq $type2);
+   foreach my $type ('estimate','approx','semi') {
+      return $type  if ($type1 eq $type  ||  $type2 eq $type);
+   }
+   return 'exact';
+}
+
 sub _calc_delta_delta {
    my($self,$delta,@args) = @_;
    my $dmt = $$self{'tz'};
    my $dmb = $$dmt{'base'};
    my $ret = $self->new_delta;
 
-   if ($self->err()) {
-      $$ret{'err'} = "[calc] First delta object invalid";
-      return $ret;
-   } elsif ($delta->err()) {
-      $$ret{'err'} = "[calc] Second delta object invalid";
+   my($subtract,$no_normalize);
+   if (@args > 2) {
+      $$ret{'err'} = "Unknown args in calc";
       return $ret;
    }
 
-   my($subtract,$no_normalize);
-   if (@args == 2) {
+   if      (@args == 2) {
       ($subtract,$no_normalize) = @args;
-   } elsif ($args[0] eq 'nonormalize') {
-      $subtract     = 0;
-      $no_normalize = 1;
+   } elsif (@args == 1) {
+      if ($args[0] eq 'nonormalize') {
+         $subtract     = 0;
+         $no_normalize = 1;
+      } else {
+         $subtract     = $args[0];
+         $no_normalize = 0;
+      }
    } else {
       $subtract     = 0;
       $no_normalize = 0;
    }
 
-   my $business = 0;
-   if ($$self{'data'}{'business'} != $$delta{'data'}{'business'}) {
+   if ($$self{'data'}{'mode'} ne $$delta{'data'}{'mode'}) {
       $$ret{'err'} = "[calc] Delta/delta calculation objects must be of " .
-        'the same type';
+        'the same mode';
       return $ret;
-   } else {
-      $business = $$self{'data'}{'business'};
    }
 
    my ($err,@delta);
@@ -865,14 +996,25 @@ sub _calc_delta_delta {
       }
    }
 
-   ($err,@delta) = $dmb->_delta_fields( { 'nonorm'  => 0,
-                                          'source'  => 'delta',
-                                          'sign'    => -1 },
-                                        [@delta])  if (! $no_normalize);
+   my $type = __type_max($$self{'data'}{'type'},
+                         $$delta{'data'}{'type'});
+   my %o = ( 'source'  => 'delta',
+             'nonorm'  => $no_normalize,
+             'sign'    => -1,
+             'type'    => $type,
+             'mode'    => $$self{'data'}{'mode'},
+           );
 
-   $$ret{'data'}{'delta'}       = [@delta];
-   $$ret{'data'}{'business'}    = $business;
-   $$self{'data'}{'length'}     = 'unknown';
+   ($err,@delta) = $dmb->_delta_fields( \%o, [@delta]);
+
+   $$ret{'data'}{'in'}         = '';
+   $$ret{'data'}{'delta'}      = [@delta];
+   $$ret{'data'}{'mode'}       = $$self{'data'}{'mode'};
+   $$ret{'data'}{'gotmode'}    = 1;
+   $$ret{'data'}{'type'}       = $type;
+   $$ret{'data'}{'type_from'}  = 'det';
+   $$ret{'data'}{'length'}     = 'unknown';
+   $$ret{'data'}{'normalized'} = 1-$no_normalize;
 
    return $ret;
 }
@@ -880,60 +1022,87 @@ sub _calc_delta_delta {
 sub convert {
    my($self,$to) = @_;
 
-   # What mode are we currently in
+   my %mode_val = ( 'exact'     => 0,
+                    'semi'      => 1,
+                    'approx'    => 2,
+                    'estimated' => 3,
+                  );
 
-   my $from;
-   my($y,$m,$w,$d,$h,$mn,$s) = @{ $$self{'data'}{'delta'} };
-   if ($y  ||  $m) {
-      $from = 'approx';
-   } elsif ($w  ||  (! $$self{'data'}{'business'}  &&  $d)) {
-      $from = 'semi';
-   } else {
-      $from = 'exact';
+   my $from     = $$self{'data'}{'type'};
+   my $from_val = $mode_val{$from};
+   my $to_val   = $mode_val{$to};
+
+   return  if ($from_val == $to_val);
+
+   #
+   # Converting from exact to less exact
+   #
+
+   if ($from_val < $to_val) {
+
+      $self->set( { 'nonorm'  => 0,
+                    'type'    => $to } );
+      return;
    }
 
-   my $business = $$self{'data'}{'business'};
-
    #
-   # Do the conversion
+   # Converting from less exact to more exact
+   # *** DEPRECATE *** 7.00
    #
 
+   my @fields;
    {
       no integer;
 
       my $dmb = $self->base();
-      my $yl  = $$dmb{'data'}{'len'}{$business}{'yl'};
-      my $ml  = $$dmb{'data'}{'len'}{$business}{'ml'};
-      my $wl  = $$dmb{'data'}{'len'}{$business}{'wl'};
-      my $dl  = $$dmb{'data'}{'len'}{$business}{'dl'};
+      my $mode= $$self{'data'}{'mode'};
+      my $yl  = $$dmb{'data'}{'len'}{$mode}{'yl'};
+      my $ml  = $$dmb{'data'}{'len'}{$mode}{'ml'};
+      my $wl  = $$dmb{'data'}{'len'}{$mode}{'wl'};
+      my $dl  = $$dmb{'data'}{'len'}{$mode}{'dl'};
 
       # Convert it to seconds
 
+      my($y,$m,$w,$d,$h,$mn,$s) = @{ $$self{'data'}{'delta'} };
       $s += $y*$yl + $m*$ml + $w*$wl + $d*$dl + $h*3600 + $mn*60;
-      ($y,$m,$w,$d,$h,$mn) = (0,0,0,0,0,0);
 
-      # Convert it to $to
+      @fields  = (0,0,0,0,0,0,$s);
 
-      if ($to eq 'approx') {
-         # Figure out how many months there are
-         $m          = int($s/$ml);
-         $s         -= $m*$ml;
-      }
+      if ($mode eq 'business') {
+
+         if ($to eq 'estimated') {
+            @fields = $dmb->_normalize_bus_est(@fields);
+
+         } elsif ($to eq 'approx'  ||
+                  $to eq 'semi') {
+            @fields = $dmb->_normalize_bus_approx(@fields);
 
-      if ($to eq 'approx'  ||  $to eq 'semi') {
-         if ($business) {
-            $w       = int($s/$wl);
-            $s      -= $w*$wl;
          } else {
-            $d       = int($s/$dl);
-            $s      -= $d*$dl;
+            @fields = $dmb->_normalize_bus_exact(@fields);
          }
-      }
 
-      $s = int($s);
+      } else {
+
+         if ($to eq 'estimated') {
+            @fields = $dmb->_normalize_est(@fields);
+
+         } elsif ($to eq 'approx'  ||
+                  $to eq 'semi') {
+            @fields = $dmb->_normalize_approx(@fields);
+
+         } else {
+            @fields = $dmb->_normalize_exact(@fields);
+         }
+
+      }
    }
 
-   $self->set('delta',[$y,$m,$w,$d,$h,$mn,$s]);
+   $$self{'data'}{'delta'}      = [ @fields ];
+   $$self{'data'}{'gotmode'}    = 1;
+   $$self{'data'}{'type'}       = $to;
+   $$self{'data'}{'type_from'}  = 'opt';
+   $$self{'data'}{'normalized'} = 1;
+   $$self{'data'}{'length'}     = 'unknown';
 }
 
 sub cmp {
@@ -953,17 +1122,17 @@ sub cmp {
       return undef;
    }
 
-   if ($$self{'data'}{'business'} != $$delta{'data'}{'business'}) {
+   if ($$self{'data'}{'mode'} ne $$delta{'data'}{'mode'}) {
       warn "WARNING: [cmp] Deltas must both be business or standard\n";
       return undef;
    }
 
-   my $business = $$self{'data'}{'business'};
-   my $dmb      = $self->base();
-   my $yl       = $$dmb{'data'}{'len'}{$business}{'yl'};
-   my $ml       = $$dmb{'data'}{'len'}{$business}{'ml'};
-   my $wl       = $$dmb{'data'}{'len'}{$business}{'wl'};
-   my $dl       = $$dmb{'data'}{'len'}{$business}{'dl'};
+   my $mode = $$self{'data'}{'mode'};
+   my $dmb  = $self->base();
+   my $yl   = $$dmb{'data'}{'len'}{$mode}{'yl'};
+   my $ml   = $$dmb{'data'}{'len'}{$mode}{'ml'};
+   my $wl   = $$dmb{'data'}{'len'}{$mode}{'wl'};
+   my $dl   = $$dmb{'data'}{'len'}{$mode}{'dl'};
 
    if ($$self{'data'}{'length'} eq 'unknown') {
       my($y,$m,$w,$d,$h,$mn,$s) = @{ $$self{'data'}{'delta'} };
@@ -981,7 +1150,7 @@ sub cmp {
                                        $d*$dl + $h*3600 + $mn*60 + $s);
    }
 
-   return ($$self{'data'}{'length'} cmp $$delta{'data'}{'length'});
+   return ($$self{'data'}{'length'} <=> $$delta{'data'}{'length'});
 }
 
 1;
