@@ -1,5 +1,5 @@
 package Date::Manip::TZ;
-# Copyright (c) 2008-2013 Sullivan Beck. All rights reserved.
+# Copyright (c) 2008-2019 Sullivan Beck. All rights reserved.
 # This program is free software; you can redistribute it and/or modify it
 # under the same terms as Perl itself.
 
@@ -22,9 +22,10 @@ use strict;
 use IO::File;
 require Date::Manip::Zones;
 use Date::Manip::Base;
+use Data::Dumper;
 
 our $VERSION;
-$VERSION='6.39';
+$VERSION='6.76';
 END { undef $VERSION; }
 
 # To get rid of a 'used only once' warnings.
@@ -98,12 +99,12 @@ sub _init {
    if ($os eq 'Unix') {
       $$self{'data'}{'path'}    = '/bin:/usr/bin';
       $$self{'data'}{'methods'} = [
-                                   qw(main TZ
-                                      env  zone TZ
-                                      file /etc/TIMEZONE
-                                      file /etc/timezone
-                                      file /etc/sysconfig/clock
-                                      file /etc/default/init
+                                   qw(main   TZ
+                                      env    zone TZ
+                                      file   /etc/TIMEZONE
+                                      file   /etc/timezone
+                                      file   /etc/sysconfig/clock
+                                      file   /etc/default/init
                                     ),
                                    'command',  '/bin/date +%Z',
                                    'command',  '/usr/bin/date +%Z',
@@ -115,6 +116,8 @@ sub _init {
                                    'command',  '/bin/date +%z',
                                    'command',  '/usr/bin/date +%z',
                                    'command',  '/usr/local/bin/date +%z',
+                                   qw( tzdata /etc/localtime /usr/share/zoneinfo
+                                    ),
                                    'gmtoff'
                                   ];
 
@@ -411,61 +414,73 @@ sub _get_curr_zone {
       my $method = shift(@methods);
       my @zone   = ();
 
+      print "*** DEBUG *** METHOD: $method ["  if ($debug);
+
       if ($method eq 'main') {
 
          if (! @methods) {
+            print "]\n"  if ($debug);
             warn "ERROR: [_set_curr_zone] main requires argument\n";
             return;
          }
          my $var = shift(@methods);
-         push(@zone,$$::var)  if (defined $$::var);
-
-         if ($debug) {
-            print "*** DEBUG ***  main $var = " .
-                  (defined $$::var ? $$::var : 'undef') . "\n";
+         print "$var] "  if ($debug);
+         no strict "refs";
+         my $val = ${ "::$var" };
+         use strict "refs";
+         if (defined $val) {
+            push(@zone,$val);
+            print "$val\n"  if ($debug);
+         } else {
+            print "undef\n"  if ($debug);
          }
 
       } elsif ($method eq 'env') {
          if (@methods < 2) {
+            print "]\n"  if ($debug);
             warn "ERROR: [_set_curr_zone] env requires 2 argument\n";
             return;
          }
          my $type = lc( shift(@methods) );
+         print "$type,"  if ($debug);
+
          if ($type ne 'zone'  &&
              $type ne 'offset') {
-            warn "ERROR: [_set_curr_zone] env requires 'offset' or 'zone' as the first argument\n";
+            print "?]\n"  if ($debug);
+            warn "ERROR: [_set_curr_zone] env requires 'offset' or 'zone' " .
+                 "as the first argument\n";
             return;
          }
          my $var  = shift(@methods);
+         print "$var] "  if ($debug);
          if (exists $ENV{$var}) {
             if ($type eq 'zone') {
                push(@zone,$ENV{$var});
+               print "$ENV{$var}\n"  if ($debug);
             } else {
                my $off = $ENV{$var};
+               print "$ENV{$var} = "  if ($debug);
                $off    = $dmb->_delta_convert('time',"0:0:$off");
                $off    = $dmb->_delta_convert('offset',$off);
+               print "$off\n"  if ($debug);
                push(@zone,$off);
             }
-         }
-
-         if ($debug) {
-            print "*** DEBUG *** env $type $var ";
-            if (exists $ENV{$var}) {
-               print $ENV{$var};
-               print $zone[$#zone]  if ($type eq 'offset');
-               print "\n";
-            } else {
-               print "-no result-\n";
-            }
+         } else {
+            print "undef\n"  if ($debug);
          }
 
       } elsif ($method eq 'file') {
          if (! @methods) {
+            print "]\n"  if ($debug);
             warn "ERROR: [_set_curr_zone] file requires argument\n";
             return;
          }
          my $file = shift(@methods);
-         next  if (! -f $file);
+         print "$file] "  if ($debug);
+         if (! -f $file) {
+            print "not found\n"  if ($debug);
+            next;
+         }
 
          my $in = new IO::File;
          $in->open($file)  ||  next;
@@ -474,13 +489,25 @@ sub _get_curr_zone {
          my @z;
          while (! $in->eof) {
             my $line = <$in>;
+            chomp($line);
             next  if ($line =~ /^\s*\043/  ||
                       $line =~ /^\s*$/);
+            if ($firstline) {
+               $firstline = 0;
+               $line      =~ s/^\s*//;
+               $line      =~ s/\s*$//;
+               $line      =~ s/["']//g;  # "
+               $line      =~ s/\s+/_/g;
+               @z         = ($line);
+            }
 
             # We're looking for lines of the form:
             #   TZ = string
             #   TIMEZONE = string
             #   ZONE = string
+            # Alternately, we may use a 1-line file (ignoring comments and
+            # whitespace) which contains only the zone name (it may be
+            # quoted or contain embedded whitespace).
             #
             # 'string' can be:
             #   the name of a timezone enclosed in single/double quotes
@@ -523,75 +550,89 @@ sub _get_curr_zone {
 
                last;
             }
-            if ($firstline) {
-               $firstline = 0;
-               $line      =~ s/^\s*//;
-               $line      =~ s/\s*$//;
-               $line      =~ s/["']//g;  # "
-               $line      =~ s/\s+/_/g;
-               push(@z,$line);
-            }
          }
          close(IN);
 
          push(@zone,@z)  if (@z);
 
          if ($debug) {
-            print "*** DEBUG *** file $file\n";
-            if (@z) {
-               print "              @z\n";
+            if (@zone) {
+               print "@zone\n";
             } else {
-               print "              -no result-\n";
+               print "no result\n";
             }
          }
 
+      } elsif ($method eq 'tzdata') {
+         if (@methods < 2) {
+            print "]\n"  if ($debug);
+            warn "ERROR: [_set_curr_zone] tzdata requires two arguments\n";
+            return;
+         }
+         my $file    = shift(@methods);
+         my $dir     = shift(@methods);
+         print "$file $dir"  if ($debug);
+
+         my $z;
+         if (-f $file  &&  -d $dir) {
+            $z = _get_zoneinfo_zone($file,$dir);
+         }
+ 	 if (defined($z)) {
+ 	    push @zone, $z;
+ 	    print "] $z\n"  if ($debug);
+ 	 } elsif ($debug) {
+ 	    print "] no result\n";
+ 	 }
+
       } elsif ($method eq 'command') {
          if (! @methods) {
+            print "]\n"  if ($debug);
             warn "ERROR: [_set_curr_zone] command requires argument\n";
             return;
          }
          my $command = shift(@methods);
+         print "$command] "  if ($debug);
          my ($out)   = _cmd($command);
          push(@zone,$out)  if ($out);
 
          if ($debug) {
-            print "*** DEBUG *** command $command\n";
             if ($out) {
-               print "              $out\n";
+               print "$out\n";
             } else {
-               print "              -no result-\n";
+               print "no output\n";
             }
          }
 
       } elsif ($method eq 'cmdfield') {
          if ($#methods < 1) {
+            print "]\n"  if ($debug);
             warn "ERROR: [_set_curr_zone] cmdfield requires 2 arguments\n";
             return;
          }
          my $command = shift(@methods);
          my $n       = shift(@methods);
+         print "$command,$n]\n"  if ($debug);
          my ($out)   = _cmd($command);
-         my @z;
+         my $val;
 
          if ($out) {
             $out    =~ s/^\s*//;
             $out    =~ s/\s*$//;
             my @out = split(/\s+/,$out);
-            push(@z,$out[$n])  if (defined $out[$n]);
+            $val    = $out[$n]  if (defined $out[$n]);
+            push(@zone,$val);
          }
 
-         push(@zone,@z)  if (@z);
-
          if ($debug) {
-            print "*** DEBUG *** cmdfield $command $n\n";
-            if (@z) {
-               print "              @z\n";
+            if ($val) {
+               print "$val\n";
             } else {
-               print "              -no result-\n";
+               print "no result\n";
             }
          }
 
       } elsif ($method eq 'gmtoff') {
+         print "] "  if ($debug);
          my($secUT,$minUT,$hourUT,$mdayUT,$monUT,$yearUT,$wdayUT,$ydayUT,
             $isdstUT) = gmtime($t);
          if ($mdayUT>($mday+1)) {
@@ -608,26 +649,27 @@ sub _get_curr_zone {
          $off    = $dmb->_delta_convert('time',"0:0:$off");
          $off    = $dmb->_delta_convert('offset',$off);
          push(@zone,$off);
-
-         if ($debug) {
-            print "*** DEBUG *** gmtoff $off\n";
-         }
+         print "$off\n"  if ($debug);
 
       } elsif ($method eq 'registry') {
+         print "] "  if ($debug);
          my $z = $self->_windows_registry_val();
-         push(@zone,$z)  if ($z);
-
-         if ($debug) {
-            print "*** DEBUG *** registry $z\n";
+         if ($z) {
+            push(@zone,$z);
+            print "$z\n"  if ($debug);
+         } else {
+            print "no result\n"  if ($debug);
          }
 
       } else {
+         print "]\n"  if ($debug);
          warn "ERROR: [_set_curr_zone] invalid method: $method\n";
          return;
       }
 
-      foreach my $zone (@zone) {
-         $zone = lc($zone);
+      while (@zone) {
+         my $zone = lc(shift(@zone));
+
          # OpenUNIX puts a colon at the start
          $zone =~ s/^://;
 
@@ -643,21 +685,95 @@ sub _get_curr_zone {
 
          # If we got an offset
 
-         $currzone = $self->zone($zone,$dstflag);
+         $currzone = $self->__zone([],'',$zone,'',$dstflag);
          last METHOD  if ($currzone);
       }
    }
 
    if (! $currzone) {
-      warn "ERROR: Date::Manip unable to determine Time Zone.\n";
-      die;
+      warn "ERROR: Date::Manip unable to determine Time Zone.  GMT will be used.\n";
+      $currzone = 'Etc/GMT';
    }
 
    return $currzone;
 }
 
-# This comes from the DateTime-TimeZone module
-#
+#######################
+# The following section comes from the DateTime-TimeZone module
+
+{
+   my $want_content;
+   my $want_size;
+   my $zoneinfo;
+
+   sub _get_zoneinfo_zone {
+      my($localtime,$z) = @_;
+      $zoneinfo = $z;
+
+      # /etc/localtime should be either a link to a tzdata file in
+      # /usr/share/zoneinfo or a copy of one of the files there.
+
+      return ''  if (! -d $zoneinfo || ! -f $localtime);
+
+      require Cwd;
+      if (-l $localtime) {
+         return _zoneinfo_file_name_to_zone(
+                                            Cwd::abs_path($localtime),
+                                            Cwd::abs_path($zoneinfo),
+                                           );
+      }
+
+      $want_content = _zoneinfo_file_slurp($localtime);
+      $want_size    = -s $localtime;
+
+      # File::Find can't bail in the middle of a find, and we only want the
+      # first match, so we'll call it in an eval.
+
+      local $@ = undef;
+      eval {
+         require File::Find;
+         File::Find::find
+             ({
+               wanted      => \&_zoneinfo_find_file,
+               no_chdir    => 1,
+              },
+              $zoneinfo,
+           );
+           1;
+      } and return;
+      ref $@
+        and return $@->{zone};
+      die $@;
+   }
+
+   sub _zoneinfo_find_file {
+      my $zone;
+      defined($zone = _zoneinfo_file_name_to_zone($File::Find::name,
+                                                  $zoneinfo))
+        and -f $_
+        and $want_size == -s _
+        and ($want_content eq _zoneinfo_file_slurp($File::Find::name))
+        and die { zone => $zone };
+   }
+}
+
+sub _zoneinfo_file_name_to_zone {
+   my($file,$zoneinfo) = @_;
+   require File::Spec;
+   my $zone = File::Spec->abs2rel($file,$zoneinfo);
+   return $zone  if (exists $Date::Manip::Zones::ZoneNames{lc($zone)});
+   return;
+}
+
+sub _zoneinfo_file_slurp {
+   my($file) = @_;
+   open my $fh, '<', $file
+     or return;
+   binmode $fh;
+   local $/ = undef;
+   return <$fh>;
+}
+
 sub _windows_registry_val {
    my($self) = @_;
 
@@ -715,6 +831,9 @@ sub _windows_registry_val {
    }
 }
 
+# End of DateTime-TimeZone section
+#######################
+
 # We will be testing commands that don't exist on all architectures,
 # so disable warnings.
 #
@@ -745,13 +864,14 @@ sub zone {
    # Parse the arguments
 
    my($zone,$abbrev,$offset,$dstflag) = ('','','','');
-   my(@abbrev,$date,$tmp);
+   my $date = [];
+   my $tmp;
    foreach my $arg (@args) {
 
       if (ref($arg) eq 'ARRAY') {
          if ($#$arg == 5) {
             # [Y,M,D,H,Mn,S]
-            return undef  if ($date);
+            return undef  if (@$date);
             $date = $arg;
 
          } elsif ($#$arg == 2) {
@@ -778,14 +898,13 @@ sub zone {
             return undef  if ($zone);
             $zone = $tmp;
 
-         } elsif (exists $$self{'data'}{'MyAbbrev'}{$arg}) {
-            return undef  if (@abbrev);
-            $abbrev = $arg;
-            @abbrev = @{ $$self{'data'}{'MyAbbrev'}{$arg} };
+         } elsif (exists $$self{'data'}{'MyAbbrev'}{$arg}  ||
+                  exists $$self{'data'}{'Abbrev'}{$arg}) {
+            return undef  if ($abbrev);
+            $abbrev       = $arg;
          } elsif (exists $$self{'data'}{'Abbrev'}{$arg}) {
-            return undef  if (@abbrev);
-            $abbrev = $arg;
-            @abbrev = @{ $$self{'data'}{'Abbrev'}{$arg} };
+            return undef  if ($abbrev);
+            $abbrev       = $arg;
 
          } elsif ($tmp = $dmb->split('offset',$arg)) {
             return undef  if ($offset);
@@ -801,6 +920,19 @@ sub zone {
       }
    }
 
+   return $self->__zone($date,$offset,$zone,$abbrev,$dstflag);
+}
+
+# $date   = [Y,M,D,H,Mn,S]
+# $offset = '-HH:Mn:SS'
+# $zone   = 'us/eastern'   (lowercase)
+# $abbrev = 'est'          (lowercase)
+# $dstflag= 'stdonly'      (lowercase)
+#
+sub __zone {
+   my($self,$date,$offset,$zone,$abbrev,$dstflag) = @_;
+   my $dmb          = $$self{'base'};
+
    #
    # Determine the zones that match all data.
    #
@@ -824,7 +956,7 @@ sub zone {
       #    zone is passed in as an offset
       #    date is passed in
 
-      $dstflag = "dst"  if ($offset  &&  $date  &&  ! $dstflag);
+      $dstflag = "dst"  if ($offset  &&  @$date  &&  ! $dstflag);
 
       my(@isdst);
       if      ($dstflag eq 'stdonly') {
@@ -837,18 +969,40 @@ sub zone {
          @isdst = (0,1);
       }
 
+      # We may pass in $zone and not $abbrev when it really should be
+      # $abbrev.
+
+      if ($zone  &&  ! $abbrev) {
+         if (exists $$self{'data'}{'Alias'}{$zone}) {
+            # no change
+         } elsif (exists $$self{'data'}{'MyAbbrev'}{$zone}  ||
+                  exists $$self{'data'}{'Abbrev'}{$zone}) {
+            $abbrev = $zone;
+            $zone   = '';
+         }
+      }
+
       # $zone
 
       if ($zone) {
-         @zone = ($zone);
+         my $z = (exists $$self{'data'}{'Alias'}{$zone} ?
+                  $$self{'data'}{'Alias'}{$zone} : $zone);
+         @zone = ($z);
       }
 
       # $abbrev
 
       if ($abbrev) {
+         my @abbrev_zones;
+         if (exists $$self{'data'}{'MyAbbrev'}{$abbrev}) {
+            @abbrev_zones = @{ $$self{'data'}{'MyAbbrev'}{$abbrev} };
+         } elsif (exists $$self{'data'}{'Abbrev'}{$abbrev}) {
+            @abbrev_zones = @{ $$self{'data'}{'Abbrev'}{$abbrev} };
+         }
+
          my @z;
          foreach my $isdst (@isdst) {
-            my @tmp = $self->_check_abbrev_isdst($abbrev,$isdst,@abbrev);
+            my @tmp = $self->_check_abbrev_isdst($abbrev,$isdst,@abbrev_zones);
             if (@tmp) {
                if (@z) {
                   @z = _list_add(\@z,\@tmp);
@@ -874,14 +1028,16 @@ sub zone {
 
          my @z;
          foreach my $isdst (@isdst) {
+            my $tmp = $$self{'data'}{'MyOffsets'}{$offset}{$isdst} ||
+                      $$self{'data'}{'Offsets'}{$offset}{$isdst};
+
             my @tmp;
-            if      (exists $$self{'data'}{'MyOffsets'}{$offset}{$isdst}) {
-               @tmp = @{ $$self{'data'}{'MyOffsets'}{$offset}{$isdst} };
-            } elsif (exists $$self{'data'}{'Offsets'}{$offset}{$isdst}) {
-               @tmp = @{ $$self{'data'}{'Offsets'}{$offset}{$isdst} };
+            if ($abbrev) {
+               @tmp = $self->_check_offset_abbrev_isdst($offset,$abbrev,$isdst,$tmp);
+            } else {
+               @tmp = @$tmp  if ($tmp);
             }
-            @tmp = $self->_check_offset_abbrev_isdst($offset,$abbrev,$isdst,@tmp)
-              if ($abbrev);
+
             if (@tmp) {
                if (@z) {
                   @z = _list_add(\@z,\@tmp);
@@ -901,7 +1057,7 @@ sub zone {
 
       # $date
 
-      if ($date) {
+      if (@$date) {
          # Get all periods for the year.
          #
          # Test all periods to make sure that $date is between the
@@ -916,17 +1072,16 @@ sub zone {
 
          ZONE:
          foreach my $z (@zone) {
-            $self->_module($z);
+            $self->_module($z)  if (! exists $$self{'data'}{'Zones'}{$z}{'Loaded'});
             my $y       = $$date[0];
             my @periods = $self->_all_periods($z,$y);
 
             foreach my $period (@periods) {
-               my($begUT,$begLT,$off,$offref,$abb,$dst,$endUT,$endLT) = @$period;
-               next  if ($dmb->cmp($date,$begLT) == -1  ||
-                         $dmb->cmp($date,$endLT) == 1 ||
-                         ($offset ne ''  &&  $offset ne $off)  ||
-                         ($isdst  ne ''  &&  $isdst  ne $dst)  ||
-                         ($abbrev ne ''  &&  lc($abbrev) ne lc($abb))
+               next  if (($abbrev ne ''  &&  lc($abbrev) ne lc($$period[4]))  ||
+                         ($offset ne ''  &&  $offset ne $$period[2])  ||
+                         ($isdst  ne ''  &&  $isdst  ne $$period[5])  ||
+                         $dmb->cmp($date,$$period[1]) == -1  ||
+                         $dmb->cmp($date,$$period[7]) == 1
                         );
                push(@tmp,$z);
                next ZONE;
@@ -962,7 +1117,7 @@ sub _check_abbrev_isdst {
    my @ret;
    ZONE:
    foreach my $zone (@zones) {
-      $self->_module($zone);
+      $self->_module($zone)  if (! exists $$self{'data'}{'Zones'}{$zone}{'Loaded'});
 
       foreach my $y (sort keys %{ $$self{'data'}{'Zones'}{$zone}{'Dates'} }) {
          my @periods = @{ $$self{'data'}{'Zones'}{$zone}{'Dates'}{$y} };
@@ -983,11 +1138,11 @@ sub _check_abbrev_isdst {
 # abbrev/isdst combination.
 #
 sub _check_offset_abbrev_isdst {
-   my($self,$offset,$abbrev,$isdst,@zones) = @_;
+   my($self,$offset,$abbrev,$isdst,$zones) = @_;
 
    my @ret;
- ZONE: foreach my $zone (@zones) {
-      $self->_module($zone);
+ ZONE: foreach my $zone (@$zones) {
+      $self->_module($zone)  if (! exists $$self{'data'}{'Zones'}{$zone}{'Loaded'});
 
       foreach my $y (sort keys %{ $$self{'data'}{'Zones'}{$zone}{'Dates'} }) {
          my @periods = @{ $$self{'data'}{'Zones'}{$zone}{'Dates'}{$y} };
@@ -1046,9 +1201,19 @@ sub all_periods {
       return;
    }
    $zone = $z;
-   $self->_module($zone);
+   $self->_module($zone)  if (! exists $$self{'data'}{'Zones'}{$zone}{'Loaded'});
 
-   return $self->_all_periods($zone,$year);
+   # Run a faster 'dclone' so we don't return the actual data.
+
+   my @tmp = $self->_all_periods($zone,$year);
+   my @ret;
+   foreach my $ele (@tmp) {
+      push(@ret,
+           [ [ @{$$ele[0]} ],[ @{$$ele[1]} ],$$ele[2],[ @{$$ele[3]} ],$$ele[4],
+             $$ele[5], [ @{$$ele[6]} ],[ @{$$ele[7]} ],$$ele[8],$$ele[9],
+             $$ele[10],$$ele[11] ]);
+   }
+   return @ret;
 }
 
 sub _all_periods {
@@ -1106,14 +1271,7 @@ sub _all_periods {
       $$self{'data'}{'Zones'}{$zone}{'AllDates'}{$year} = [ @periods ];
    }
 
-   # A faster 'dclone' so we don't return the actual data
-   my @ret;
-   foreach my $ele (@{ $$self{'data'}{'Zones'}{$zone}{'AllDates'}{$year} }) {
-      push(@ret,
-           [ [ @{$$ele[0]} ],[ @{$$ele[1]} ],$$ele[2],[ @{$$ele[3]} ],$$ele[4],$$ele[5],
-             [ @{$$ele[6]} ],[ @{$$ele[7]} ],$$ele[8],$$ele[9],$$ele[10],$$ele[11] ]);
-   }
-   return @ret;
+   return @{ $$self{'data'}{'Zones'}{$zone}{'AllDates'}{$year} };
 }
 
 sub periods {
@@ -1125,7 +1283,7 @@ sub periods {
       return;
    }
    $zone = $z;
-   $self->_module($zone);
+   $self->_module($zone)  if (! exists $$self{'data'}{'Zones'}{$zone}{'Loaded'});
 
    if (! defined($year1)) {
       return $self->_periods($zone,$year);
@@ -1191,12 +1349,13 @@ sub date_period {
       return;
    }
    $zone = $z;
-   $self->_module($zone);
+   $self->_module($zone)  if (! exists $$self{'data'}{'Zones'}{$zone}{'Loaded'});
 
    my $dmb  = $$self{'base'};
    my @date = @$date;
    my $year = $date[0];
    my $dates= $dmb->_date_fields(@$date);
+   return ()  if ($year < 0  ||  $year > 9999);
 
    if ($wallclock) {
       # A wallclock date
@@ -1480,17 +1639,37 @@ sub _convert {
 # REGULAR EXPRESSIONS FOR TIMEZONE INFORMATION
 ########################################################################
 
-# Returns a regular expression capable of matching all timezone names
-# and aliases.
+# Returns regular expressions capable of matching timezones.
+#
+# The timezone regular expressions are:
+#   namerx   : this will match a zone name or alias (America/New_York)
+#   abbrx    : this will match a zone abbreviation (EDT)
+#   zonerx   : this will match a zone name or an abbreviation
+#   offrx    : this will match a pure offset (+0400)
+#   offabbrx : this will match an offset with an abbreviation (+0400 WET)
+#   offparrx : this will match an offset and abbreviation if parentheses
+#              ("+0400 (WET)")
+#   zrx      : this will match all forms
 #
 # The regular expression will have the following named matches:
-#   zone  = a zone name or alias
+#   tzstring : the full string matched
+#   zone     : the name/alias
+#   abb      : the zone abbrevation
+#   off      : the offset
 #
-sub _zonerx {
-   my($self) = @_;
-   return $$self{'data'}{'zonerx'}  if (defined $$self{'data'}{'zonerx'});
-   my @zone  = (keys %{ $$self{'data'}{'Alias'} },
+sub _zrx {
+   my($self,$re) = @_;
+   return $$self{'data'}{$re}  if (defined $$self{'data'}{$re});
+
+   # Zone name
+
+   my @zone;
+   if (exists $ENV{'DATE_MANIP_DEBUG_ZONES'}) {
+      @zone  = split(/\s+/,$ENV{'DATE_MANIP_DEBUG_ZONES'});
+   } else {
+      @zone  = (keys %{ $$self{'data'}{'Alias'} },
                 keys %{ $$self{'data'}{'MyAlias'} });
+   }
    @zone     = sort _sortByLength(@zone);
    foreach my $zone (@zone) {
       $zone  =~ s/\057/\\057/g;   # /
@@ -1500,83 +1679,61 @@ sub _zonerx {
       $zone  =~ s/\051/\\051/g;   # )
       $zone  =~ s/\053/\\053/g;   # +
    }
-   my $re    = join('|',@zone);
-   $$self{'data'}{'zonerx'} = qr/(?<zone>$re)/i;
-   return $$self{'data'}{'zonerx'};
-}
 
-# Returns a regular expression capable of matching all abbreviations.
-#
-# The regular expression will have the following named matches:
-#   abb  = a zone abbreviation
-#
-sub _abbrx {
-   my($self) = @_;
-   return $$self{'data'}{'abbrx'}  if (defined $$self{'data'}{'abbrx'});
-   my @abb  = (keys %{ $$self{'data'}{'Abbrev'} },
+   my $zone  = join('|',@zone);
+   $zone     = qr/(?<zone>$zone)/i;
+
+   # Abbreviation
+
+   my @abb;
+   if (exists $ENV{'DATE_MANIP_DEBUG_ABBREVS'}) {
+      @abb  = split(/\s+/,$ENV{'DATE_MANIP_DEBUG_ABBREVS'});
+   } else {
+      @abb  = (keys %{ $$self{'data'}{'Abbrev'} },
                keys %{ $$self{'data'}{'MyAbbrev'} });
+   }
    @abb     = sort _sortByLength(@abb);
    foreach my $abb (@abb) {
-      $abb =~ s/\055/\\055/g;   # -
-      $abb =~ s/\053/\\053/g;   # +
+      $abb  =~ s/\055/\\055/g;   # -
+      $abb  =~ s/\053/\\053/g;   # +
    }
-   my $re    = join('|',@abb);
-   $$self{'data'}{'abbrx'} = qr/(?<abb>$re)/i;
-   return $$self{'data'}{'abbrx'};
-}
 
-# Returns a regular expression capable of matching a valid timezone as
-# an offset. Known formats are:
-#    +07              +07 (HST)
-#    +0700            +0700 (HST)
-#    +07:00           +07:00 (HST)
-#    +070000          +070000 (HST)
-#    +07:00:00        +07:00:00 (HST)
-#
-# The regular expression will have the following named matches:
-#   off   = the offset
-#   abb   = the abbreviation
-#
-sub _offrx {
-   my($self) = @_;
-   return $$self{'data'}{'offrx'}  if (defined $$self{'data'}{'offrx'});
+   my $abb  = join('|',@abb);
+   $abb     = qr/(?<abb>$abb)/i;
 
-   my($hr) = qr/(?:[0-1][0-9]|2[0-3])/;  # 00 - 23
-   my($mn) = qr/(?:[0-5][0-9])/;         # 00 - 59
-   my($ss) = qr/(?:[0-5][0-9])/;         # 00 - 59
-   my($abb)= $self->_abbrx();
+   # Offset (+HH, +HHMM, +HH:MM, +HH:MM:SS, +HHMMSS)
 
-   my($re) = qr/ (?<off> [+-] (?: $hr:$mn:$ss |
-                                  $hr$mn$ss   |
-                                  $hr:?$mn    |
-                                  $hr
-                              )
-                 )
-                 (?: \s* (?: \( $abb \) | $abb))? /ix;
+   my($hr)  = qr/(?:[0-1][0-9]|2[0-3])/;  # 00 - 23
+   my($mn)  = qr/(?:[0-5][0-9])/;         # 00 - 59
+   my($ss)  = qr/(?:[0-5][0-9])/;         # 00 - 59
 
-   $$self{'data'}{'offrx'} = $re;
-   return $$self{'data'}{'offrx'};
-}
+   my($off) = qr/ (?<off> [+-] (?: $hr:$mn:$ss |
+                                   $hr$mn$ss   |
+                                   $hr:?$mn    |
+                                   $hr
+                               )
+                  ) /ix;
 
-# Returns a regular expression capable of matching all timezone
-# information available. It will match a full timezone, an
-# abbreviation, or an offset/abbreviation combination. The regular
-# expression will have the following named matches:
-#    tzstring  = the full string matched
-# in addition to the matches from the _zonerx, _abbrx, and _offrx
-# functions.
-#
-sub _zrx {
-   my($self) = @_;
-   return $$self{'data'}{'zrx'}  if (defined $$self{'data'}{'zrx'});
+   # Assemble everything
+   #
+   # A timezone can be any of the following in this order:
+   #    Offset (ABB)
+   #    Offset ABB
+   #    ABB
+   #    Zone
+   #    Offset
+   # We put ABB before Zone so CET gets parse as the more common abbreviation
+   # than the less common zone name.
 
-   my $zonerx    = $self->_zonerx();          # (?<zone>america/new_york|...)
-   my $zoneabbrx = $self->_abbrx();           # (?<abb>edt|est|...)
-   my $zoneoffrx = $self->_offrx();           # (?<off>07:00) (?<abb>GMT)
+   $$self{'data'}{'namerx'}   = qr/(?<tzstring>$zone)/;
+   $$self{'data'}{'abbrx'}    = qr/(?<tzstring>$abb)/;
+   $$self{'data'}{'zonerx'}   = qr/(?<tzstring>(?:$abb|$zone))/;
+   $$self{'data'}{'offrx'}    = qr/(?<tzstring>$off)/;
+   $$self{'data'}{'offabbrx'} = qr/(?<tzstring>$off\s+$abb)/;
+   $$self{'data'}{'offparrx'} = qr/(?<tzstring>$off\s*\($abb\))/;
+   $$self{'data'}{'zrx'}      = qr/(?<tzstring>(?:$off\s*\($abb\)|$off\s+$abb|$abb|$zone|$off))/;
 
-   my $zrx       = qr/(?<tzstring>$zoneabbrx|$zoneoffrx|$zonerx)/;
-   $$self{'data'}{'zrx'} = $zrx;
-   return $zrx;
+   return $$self{'data'}{$re};
 }
 
 # This sorts from longest to shortest element
@@ -1721,11 +1878,9 @@ sub _config_var_setdate {
 
    if ($zone) {
       my ($err,@args);
-      push(@args,$date)  if ($date);
-      push(@args,$zone);
-      push(@args,$dstflag);
-
-      $zone = $self->zone(@args);
+      my $dmb = $$self{'base'};
+      $date = []  if (! defined $date);
+      $zone = $self->__zone($date,'',lc($zone),'',lc($dstflag));
       if (! $zone) {
          warn "ERROR: [config_var] invalid zone in SetDate: @args\n";
          return 1;
